/******************************************************************

 Copyright 2001 by Object Craft P/L, Melbourne, Australia.

 LICENCE - see LICENCE file distributed with this software for details.

******************************************************************/

#include "mssqldb.h"

static int has_been_freed(LOGINRECObj *self)
{
    if (self->login == NULL) {
        PyErr_SetString(PyExc_TypeError, "LOGINREC has been freed");
	return 1;
    }
    return 0;
}

static char LOGINREC_dbfreelogin__doc__[] = 
"dbfreelogin() -> None";

static PyObject *LOGINREC_dbfreelogin(LOGINRECObj *self, PyObject *args)
{
    if (!PyArg_ParseTuple(args, "") || has_been_freed(self))
	return NULL;

    PyErr_Clear();
#ifdef _WIN32
    dbfreelogin(self->login);
    if (self->debug)
	debug_msg("dbfreelogin()\n");
#else
    dbloginfree(self->login);
    if (self->debug)
	debug_msg("dbloginfree()\n");
#endif
    if (PyErr_Occurred())
	return NULL;

    self->login = NULL;
    Py_INCREF(Py_None);
    return Py_None;
}

static char LOGINREC_dbopen__doc__[] = 
"dbopen(server) -> DBPROC";

static PyObject *LOGINREC_dbopen(LOGINRECObj *self, PyObject *args)
{
    char *server = NULL;

    if (!PyArg_ParseTuple(args, "|s", &server) || has_been_freed(self))
	return NULL;
    return DBPROC__init__(self->login, server, self->debug);
}

static char LOGINREC_dbsetlapp__doc__[] = 
"dbsetlapp(appname) -> bool";

static PyObject *LOGINREC_dbsetlapp(LOGINRECObj *self, PyObject *args)
{
    char *appname;
    RETCODE status;

    if (!PyArg_ParseTuple(args, "s", &appname) || has_been_freed(self))
	return NULL;

    PyErr_Clear();
    status = DBSETLAPP(self->login, appname);
    if (self->debug)
	debug_msg("DBSETLAPP('%s') -> %s\n",
		  appname, value_str(TYPE_RETCODE, status));
    if (PyErr_Occurred())
	return NULL;

    return PyInt_FromLong(status);
}

static char LOGINREC_dbsetlhost__doc__[] = 
"dbsetlhost(hostname) -> bool";

static PyObject *LOGINREC_dbsetlhost(LOGINRECObj *self, PyObject *args)
{
    char *host;
    RETCODE status;

    if (!PyArg_ParseTuple(args, "s", &host) || has_been_freed(self))
	return NULL;

    PyErr_Clear();
    status = DBSETLHOST(self->login, host);
    if (self->debug)
	debug_msg("DBSETLHOST('%s') -> %s\n",
		  host, value_str(TYPE_RETCODE, status));
    if (PyErr_Occurred())
	return NULL;

    return PyInt_FromLong(status);
}

static char LOGINREC_dbsetlpwd__doc__[] = 
"dbsetlpwd(password) -> bool";

static PyObject *LOGINREC_dbsetlpwd(LOGINRECObj *self, PyObject *args)
{
    char *passwd;
    RETCODE status;

    if (!PyArg_ParseTuple(args, "s", &passwd) || has_been_freed(self))
	return NULL;

    PyErr_Clear();
    status = DBSETLPWD(self->login, passwd);
    if (self->debug)
	debug_msg("DBSETLPWD('%s') -> %s\n",
		  passwd, value_str(TYPE_RETCODE, status));
    if (PyErr_Occurred())
	return NULL;

    return PyInt_FromLong(status);
}

static char LOGINREC_dbsetluser__doc__[] = 
"dbsetluser(username) -> bool";

static PyObject *LOGINREC_dbsetluser(LOGINRECObj *self, PyObject *args)
{
    char *user;
    RETCODE status;

    if (!PyArg_ParseTuple(args, "s", &user) || has_been_freed(self))
	return NULL;

    PyErr_Clear();
    status = DBSETLUSER(self->login, user);
    if (self->debug)
	debug_msg("DBSETLUSER('%s') -> %s\n",
		  user, value_str(TYPE_RETCODE, status));
    if (PyErr_Occurred())
	return NULL;

    return PyInt_FromLong(status);
}

static struct PyMethodDef LOGINREC_methods[] = {
    { "dbfreelogin", (PyCFunction)LOGINREC_dbfreelogin, METH_VARARGS, LOGINREC_dbfreelogin__doc__ },
    { "dbopen", (PyCFunction)LOGINREC_dbopen, METH_VARARGS, LOGINREC_dbopen__doc__ },
    { "dbsetlapp", (PyCFunction)LOGINREC_dbsetlapp, METH_VARARGS, LOGINREC_dbsetlapp__doc__ },
    { "dbsetlhost", (PyCFunction)LOGINREC_dbsetlhost, METH_VARARGS, LOGINREC_dbsetlhost__doc__ },
    { "dbsetlpwd", (PyCFunction)LOGINREC_dbsetlpwd, METH_VARARGS, LOGINREC_dbsetlpwd__doc__ },
    { "dbsetluser", (PyCFunction)LOGINREC_dbsetluser, METH_VARARGS, LOGINREC_dbsetluser__doc__ },
    { NULL, NULL }
};

PyObject *LOGINREC__init__(int debug)
{
    LOGINREC *login;
    LOGINRECObj *self;

    login = dblogin();
    if (debug)
	debug_msg("dblogin() -> OK\n");
    if (login == NULL) {
	PyErr_SetString(PyExc_SystemError, "dblogin failed");
	return NULL;
    }

    self = PyObject_NEW(LOGINRECObj, &LOGINRECType);
    if (self == NULL)
	return NULL;
    self->login = login;
    self->debug = debug;
    return (PyObject *)self;
}

static void LOGINREC__del__(LOGINRECObj *self)
{
    if (self->login != NULL) {
#ifdef _WIN32
	dbfreelogin(self->login);
	if (self->debug)
	    debug_msg("dbfreelogin()\n");
#else
	dbloginfree(self->login);
	if (self->debug)
	    debug_msg("dbloginfree()\n");
#endif
    }
    PyMem_DEL(self);
}

#define OFFSET(x) offsetof(LOGINRECObj, x)

static struct memberlist LOGINREC_memberlist[] = {
    { "debug", T_INT, OFFSET(debug) },
    { NULL }
};

static PyObject *LOGINREC_getattr(LOGINRECObj *self, char *name)
{
    PyObject *rv;

    rv = PyMember_Get((char *)self, LOGINREC_memberlist, name);
    if (rv)
	return rv;
    PyErr_Clear();
    return Py_FindMethod(LOGINREC_methods, (PyObject *)self, name);
}

static int LOGINREC_setattr(LOGINRECObj *self, char *name, PyObject *v)
{
    if (v == NULL) {
	PyErr_SetString(PyExc_AttributeError, "Cannot delete attribute");
	return -1;
    }
    return PyMember_Set((char *)self, LOGINREC_memberlist, name, v);
}

static char LOGINRECType__doc__[] = 
"";

PyTypeObject LOGINRECType = {
    PyObject_HEAD_INIT(0)
    0,				/*ob_size*/
    "LOGINREC",			/*tp_name*/
    sizeof(LOGINRECObj),	/*tp_basicsize*/
    0,				/*tp_itemsize*/
    /* methods */
    (destructor)LOGINREC__del__,/*tp_dealloc*/
    (printfunc)0,		/*tp_print*/
    (getattrfunc)LOGINREC_getattr, /*tp_getattr*/
    (setattrfunc)LOGINREC_setattr, /*tp_setattr*/
    (cmpfunc)0,			/*tp_compare*/
    (reprfunc)0,		/*tp_repr*/
    0,				/*tp_as_number*/
    0,				/*tp_as_sequence*/
    0,				/*tp_as_mapping*/
    (hashfunc)0,		/*tp_hash*/
    (ternaryfunc)0,		/*tp_call*/
    (reprfunc)0,		/*tp_str*/

    0L, 0L, 0L, 0L,
    LOGINRECType__doc__
};
